@@ -2,11 +2,13 @@
 
 use std::{
     collections::{HashMap, HashSet},
+    ffi::{c_int, c_long},
     ops::{BitAndAssign, BitOrAssign, BitXorAssign},
     str::FromStr,
 };
 
 use either::Either;
+use kadmin_sys::{krb5_enctype, krb5_flags, krb5_int16, krb5_int32, krb5_octet};
 use pyo3::{
     prelude::*,
     types::{PyDict, PyString, PyTuple},
@@ -15,7 +17,7 @@ use pyo3::{
 use crate::{
     db_args::DbArgs,
     error::Result,
-    kadmin::{KAdminApiVersion, KAdminImpl},
+    kadmin::{KAdminApiVersion, KAdminImpl, KAdminPrivileges},
     keysalt::{EncryptionType, KeySalt, KeySalts, SaltType},
     params::Params,
     policy::Policy,
@@ -41,6 +43,7 @@ fn init(m: &Bound<'_, PyModule>) -> PyResult<()> {
     m.add_class::<PyPrincipalBuilderKey>()?;
     m.add_class::<Principal>()?;
     m.add_class::<Policy>()?;
+    m.add_class::<KAdminPrivileges>()?;
     exceptions::init(m)?;
     Ok(())
 }
@@ -52,8 +55,8 @@ impl Params {
     #[allow(clippy::too_many_arguments)]
     fn py_new(
         realm: Option<&str>,
-        kadmind_port: Option<i32>,
-        kpasswd_port: Option<i32>,
+        kadmind_port: Option<c_int>,
+        kpasswd_port: Option<c_int>,
         admin_server: Option<&str>,
         dbname: Option<&str>,
         acl_file: Option<&str>,
@@ -125,7 +128,7 @@ impl DbArgs {
 #[pymethods]
 impl EncryptionType {
     #[new]
-    fn py_new(enctype: Either<i32, String>) -> Result<Self> {
+    fn py_new(enctype: Either<krb5_enctype, String>) -> Result<Self> {
         Ok(match enctype {
             Either::Left(i) => i.try_into()?,
             Either::Right(s) => EncryptionType::from_str(&s)?,
@@ -137,7 +140,7 @@ impl EncryptionType {
 impl SaltType {
     #[new]
     #[pyo3(signature = (salttype = None))]
-    fn py_new(salttype: Option<Either<i32, String>>) -> Result<Self> {
+    fn py_new(salttype: Option<Either<krb5_int32, String>>) -> Result<Self> {
         Ok(match salttype {
             None => Default::default(),
             Some(salttype) => match salttype {
@@ -171,7 +174,7 @@ impl KeySalts {
 #[pymethods]
 impl TlDataEntry {
     #[new]
-    fn py_new(data_type: i16, contents: Vec<u8>) -> Self {
+    fn py_new(data_type: krb5_int16, contents: Vec<krb5_octet>) -> Self {
         Self {
             data_type,
             contents,
@@ -246,12 +249,12 @@ impl PrincipalAttributes {
     const PySupportDesMd5: Self = Self::SupportDesMd5;
 
     #[new]
-    fn py_new(bits: i32) -> Self {
+    fn py_new(bits: krb5_flags) -> Self {
         Self::from_bits_retain(bits)
     }
 
     #[pyo3(name = "bits")]
-    fn py_bits(&self) -> i32 {
+    fn py_bits(&self) -> krb5_flags {
         self.bits()
     }
 
@@ -287,7 +290,7 @@ impl PrincipalAttributes {
         self.complement()
     }
 
-    fn __int__(&self) -> i32 {
+    fn __int__(&self) -> krb5_flags {
         self.bits()
     }
 }
@@ -492,6 +495,11 @@ impl KAdmin {
         self.list_policies(query)
     }
 
+    #[pyo3(name = "get_privileges")]
+    fn py_get_privileges(&self) -> Result<KAdminPrivileges> {
+        self.get_privileges()
+    }
+
     #[cfg(feature = "client")]
     #[staticmethod]
     #[pyo3(name = "with_password", signature = (client_name, password, params=None, db_args=None, api_version=None))]
@@ -723,9 +731,73 @@ impl Policy {
     }
 }
 
+#[pymethods]
+#[allow(non_upper_case_globals)]
+impl KAdminPrivileges {
+    #[classattr]
+    #[pyo3(name = "Add")]
+    const PyAdd: Self = Self::Add;
+    #[classattr]
+    #[pyo3(name = "Delete")]
+    const PyDelete: Self = Self::Delete;
+    #[classattr]
+    #[pyo3(name = "Inquire")]
+    const PyInquire: Self = Self::Inquire;
+    #[classattr]
+    #[pyo3(name = "Modify")]
+    const PyModify: Self = Self::Modify;
+
+    #[new]
+    fn py_new(bits: c_long) -> Self {
+        Self::from_bits_retain(bits)
+    }
+
+    #[pyo3(name = "bits")]
+    fn py_bits(&self) -> c_long {
+        self.bits()
+    }
+
+    fn __contains__(&self, other: Self) -> bool {
+        self.contains(other)
+    }
+
+    fn __and__(&self, other: Self) -> Self {
+        *self & other
+    }
+
+    fn __iand__(&mut self, other: Self) {
+        Self::bitand_assign(self, other);
+    }
+
+    fn __xor__(&self, other: Self) -> Self {
+        *self ^ other
+    }
+
+    fn __ixor__(&mut self, other: Self) {
+        Self::bitxor_assign(self, other);
+    }
+
+    fn __or__(&self, other: Self) -> Self {
+        *self | other
+    }
+
+    fn __ior__(&mut self, other: Self) {
+        Self::bitor_assign(self, other);
+    }
+
+    fn __invert__(&self) -> Self {
+        self.complement()
+    }
+
+    fn __int__(&self) -> c_long {
+        self.bits()
+    }
+}
+
 /// python-kadmin-rs exceptions
 mod exceptions {
     use indoc::indoc;
+    use kadmin_sys::kadm5_ret_t;
     use pyo3::{create_exception, exceptions::PyException, intern, prelude::*};
 
     use crate::error::Error;
@@ -862,7 +934,7 @@ mod exceptions {
             let (exc, extras) = match &error {
                 Error::Kerberos { code, message } => (
                     KerberosException::new_err(error.to_string()),
-                    Some((*code as i64, message)),
+                    Some((*code as kadm5_ret_t, message)),
                 ),
                 Error::KAdmin { code, message } => (
                     KAdminException::new_err(error.to_string()),
